"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.filterArchives = exports.isArchive = exports.isZip = exports.isTar = exports.extract = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const common_1 = require("./common");
const debug_1 = require("./debug");
const pMap = require('p-map');
const AdmZip = require('adm-zip');
const tar = require('tar');
const { mkdir } = fs_1.promises;
const zipFormats = ['.zip', '.zipx'];
const tarFormats = ['.tar', '.gz', '.tgz'];
async function handleExtraction(path, temporaryDir, childArchiveHandler) {
    const extractionTarget = path_1.join(temporaryDir, path.includes(temporaryDir)
        ? path_1.relative(temporaryDir, `${path}${common_1.EXTRACTED_DIR_SUFFIX}`)
        : path_1.basename(path));
    await mkdir(extractionTarget, { recursive: true });
    if (isTar(path)) {
        await tar.x({
            file: path,
            cwd: extractionTarget,
            sync: true,
            onentry: (entry) => {
                const childAbsolutePath = path_1.join(extractionTarget, entry.path);
                if (isArchive(childAbsolutePath)) {
                    childArchiveHandler(childAbsolutePath);
                }
            },
        });
    }
    else if (isZip(path)) {
        const zip = new AdmZip(path);
        await pMap(zip.getEntries(), (entry) => {
            const childAbsolutePath = path_1.join(extractionTarget, entry.entryName);
            zip.extractEntryTo(entry.entryName, extractionTarget, true, true);
            if (isArchive(childAbsolutePath)) {
                childArchiveHandler(childAbsolutePath);
            }
        }, { concurrency: common_1.DECOMPRESSING_CONCURRENCY_LEVEL });
    }
}
async function extract(archives, temporaryDir, depthLimit, depth = 0) {
    if (depth >= depthLimit) {
        return;
    }
    const childArchives = [];
    for (const archive of archives) {
        try {
            await handleExtraction(archive, temporaryDir, (childArchive) => childArchives.push(childArchive));
        }
        catch (err) {
            debug_1.debug(`Could not extract archive: ${archive} ${err}`);
        }
    }
    if (childArchives.length > 0) {
        await extract(childArchives, temporaryDir, depthLimit, depth + 1);
    }
}
exports.extract = extract;
function isTar(path) {
    return tarFormats.includes(path_1.extname(path));
}
exports.isTar = isTar;
function isZip(path) {
    return zipFormats.includes(path_1.extname(path));
}
exports.isZip = isZip;
function isArchive(path) {
    return isTar(path) || isZip(path);
}
exports.isArchive = isArchive;
function filterArchives(paths) {
    return paths.filter((path) => isArchive(path));
}
exports.filterArchives = filterArchives;
//# sourceMappingURL=extract.js.map